<?php 

  /**
   * (c) David Smith, 2008
   * nopeer@gmail.com
   *
   *
   * THE PROBLEM
   *
   *     The most basic form of attack on a system, is with escape sequence data
   *     being sent using the normal methods of input such as _GET and _POST.
   *
   *     Escape sequence functions such as mysql_real_escape_string give us
   *     the ability to prevent bad/maliscious data from breaking/compromising
   *     systems. 
   *
   *     How these functions are used depends upon your design. In the worst
   *     case you can call mysql_real_escape_string on each string as you add it
   *     to a mysql_query call. This is both repetitious and pointless as you only
   *     have to miss it once to compromise your system. A better design, will
   *     have queries encapsulated in a function or class and you can make your
   *     call to the escape function once. However this requires that you pass
   *     each argument for the query separately. Which in most cases would
   *     rquire a function with a variable number of parameters.
   *
   *     The best approach is to escape all
   *     strings as they are received, eg calling mysql_real_escape string on
   *     every string under the _GET and _POST arrays and storing them under a
   *     new array. Any later use of _GET or _POST in code can be detected with
   *     a simple grep.
   *
   *     One of the biggest headaches with all of this is knowing what state a
   *     string is in. If you escape a string twice, you have introduced unwanted data.
   *         eg a ' becomes \' and then \\\'
   *
   *     If you send an escaped string to the database, then retrieve it it will
   *     no longer be escaped. You are constantly having to figure out what state
   *     a string is in before you use it.
   *
   * THE SOLUTION
   *
   *     The best way to solve these problems is by establishing a policy. As always
   *     it is up to the programmer to follow/ignore a policy. The policy that I have
   *     chosen is to create a GREEN ZONE within my programs. Every string inside that
   *     zone is to be properly escaped at all times. To facilitate this, I have constructed
   *     this class to create a GREEN WALL. All data entering and leaving the GREEN ZONE
   *     must pass through the GREEN WALL. The wall will ensure that all data is properly
   *     escaped when entering the zone, and escaped or not escaped when leaving depending
   *     on where it is going ( output to browser should not be escaped, output to database
   *     should be ).
   *
   *     The choice as to wether code should be escaped or not escaped within the zone
   *     is arbitrary, I could have chosen to have all strings not escaped within the zone.
   *     Which would be like building a zone around the database. But databases are not
   *     the only problems. php functions like eval, are also susceptible to escape sequence
   *     attacks, therefore all calls to eval etc would have to pass through the wall also.
   *     So it is better, to have the zone pretty much encompass the whole php program. Which
   *     is what this class attempts to acheive.
   *
   *     Since I use postgres or mysql databases and smarty for outputting. The GreenWall
   *     that I have constructed encapsulates these interfaces. Where possible I have made
   *     the interfaces identical, so that programmers don't have to learn too much in order
   *     to work with the green wall.
   *
   *     smarty = new Smarty();           becomes      smarty = new GreenWall->NewSmarty();
   *     smarty->display( 'my.tpl' );                  smarty->display( 'my.tpl' );  
   *
   *     mysql_query( ... );              becomes      GreenWall->mysql_query( ... );
   *     mysql_fetch_row( ... );
   *
   *     One last note, there is a difference between addslashes and mysql_real_escape_string,
   *     you can read about one definite example here http://shiflett.org/archive/184
   **/



require_once GREEN_ZONE_DIR . 'Utils/MysqlDB.php';




class GreenZone extends MysqlDB
{
  public $_GET;
  public $_POST;

  /**
   * Constructor
   *
   *    DOES     - sets up the database aspect and stores the get and post data in new arrays
   *    RECEIVES - Nothing
   *    RETURNS  - Nothing
   *    THROWS   - Nothing
   **/

  function __construct()
  {
    parent::__construct();
    
    global $_POST;
    global $_GET;

    if( !get_magic_quotes_gpc() )
    {
      $this->_GET = $this->SecureArray( $_GET );
      $this->_POST = $this->SecureArray( $_POST );
    }
    else
    {
      $this->_GET = $_GET;
      $this->_POST = $_POST;
    }
      
    $log = null;
  }


  function get( $target )
  {
    return $this->_GET[$target];
  }

  function post( $target )
  {
    return $this->_POST[$target];
  }


  /**
   * SecureArray 
   *
   *    DOES     - This function converts all strings in an array using mysql_real_escape_string.
   *               It will also branch down to arrays in arrays.
   *    RECEIVES - The array to secure
   *    RETURNS  - A copy of the array that is secured
   *    THROWS   - Nothing
   **/
  
  function SecureArray( $source_array )
  {
    $result_array = array();
    
    if( is_array( $source_array ) )
      {
	foreach( $source_array as $source_name => $source_value )
	  {
	    if( is_array( $source_value ) )
	      {
		$result_array[ $source_name ] = SecureArray( $source_value );
	      }
	    else if( is_string( $source_value ) )
	      {
		$result_array[ $source_name ] = mysql_real_escape_string( $source_value );
	      }
	    else
	      {
		$result_array[ $source_name ] = $source_value;
	      }
	  }
      }
    
    return $result_array;
  }



  /**
   * ReleaseArray - remove the slashes from all the strings under an array.
   **/

  function ReleaseArray( $source_array )
  {
    $result_array = array();
    
    if( is_array( $source_array ) )
      {
	foreach( $source_array as $source_name => $source_value )
	  {
	    if( is_array( $source_value ) )
	      {
		$result_array[ $source_name ] = $this->ReleaseArray( $source_value );
	      }
	    else if( is_string( $source_value ) )
	      {
		$result_array[ $source_name ] = stripslashes( $source_value );
	      }
	    else
	      {
		$result_array[ $source_name ] = $source_value;
	      }
	  }
      }
    
    return $result_array;
  }







  /**
   * Wrapper for mysql_fetch_array
   **/

  function mysql_fetch_array( $result_set, $result_type = MYSQL_BOTH )
  {
    // Call fetch array to get the result set
    // if we received a row as a result.
    //     for each element in the row
    //         if the element is a string
    //             escape the string.
    // return the row

    $row = mysql_fetch_array( $result_set, $result_type );

    if( $row !== FALSE )
      {
	foreach( $row as $next_name => $next_value )
	  {
	    if( is_string( $next_value ) )
	      {
		$row[ $next_name ] = mysql_real_escape_string( $next_value );
	      }
	  }
      }

    return $row;
  }



  /**
   * smarty_assign - Wrapper for smarty->assign
   **/

  function smarty_assign( $smarty_object, $template_varname, $value )
  {
    $adjusted_value = '';

    if( is_string( $value ) )
      {
	$adjusted_value = stripslashes( $value );
      }
    else if ( is_array( $value ) )
      {
	$adjusted_value = $this->ReleaseArray( $value );
      }
    else
      {
	$adjusted_value = $value;
      }

    $smarty_object->assign( $template_varname, $adjusted_value );
  }



  /**
   * EnsureSecure
   *
   *    DOES     - Checks this projects code that violates the green zone.
   *    RECEIVES - Nothing
   *    RETURNS  - Nothing
   *    THROWS   - A GreenZoneException "Green Zone insecure"
   *    TODO     - implement this
   **/

  function EnsureSecure()
  {
    // echo 'No real checking yet, so thumbs up <br>';
  }





  /**
   * CreateSmarty 
   *
   *    DOES     - This function creates a smarty object. Its just here so we can encapsulate
   *               the calls to configure it.
   *    RECEIVES - Nothing
   *    RETURNS  - A new smarty object with its compile and template directories set.
   *    THROWS   - Nothing
   **/
  
  function CreateSmarty()
  {
    $smarty = new Smarty();

    $smarty->template_dir = SMARTY_TEMPLATES_DIR;
    $smarty->compile_dir = SMARTY_COMPILE_DIR;
    
    return $smarty;
  }




  /**
   * ValidPassword
   *
   *    DOES     - Checks that the value given is a string greater than 7 characters.
   *    RECEIVES - The password to test
   *    RETURNS  - true or false, valid or invalid
   *    THROWS   - Nothing
   **/
  
  function ValidPassword( $target_password )
  {
    if( !is_string( $target_password ) )
      {
	SetLastError( 'Password needs to be a string' );
	return false;
      }
    else if( strlen( $target_password ) < 8 )
      {
	SetLastError( 'Password must be atleast 8 characters' );
	return false;
    }
    
    return true;
  }


/**
 * SetupLogFile
 * 
 *    DOES     - Creates a GZ_Logger object to handle logging requests
 *    RECEIVES - The path to the log file
 *    RETURNS  - Nothing
 *    THROWS   - Nothing, an exception will be called when you try to write
 *               an error to the file if its not writeable
 **/
  
  public function SetupLogFile( $log_filename )
  {
    $this->log = new GZ_Logger( $log_filename );
  }


  
  /**
  * LogMessage
  * 
  *    DOES     - Write a message to the logfile
  *    RECEIVES - An identifyer for the user, the main message, and some variable parameters
  *    RETURNS  - Nothing
  *    THROWS   - The GZ_Logger will throw an exception if it can't write to the log file
  **/
  
  public function LogMessage( $user, $message, $param1, $param2, $param3 )
  {
    $this->log->LogMessage( $user, $message, $param1, $param2, $param3 );
  }


// PRIVATE
    
  private $log;


};







?>
